"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const arg_1 = __importDefault(require("arg"));
const filter = __importStar(require("../utils/filter"));
const factory_1 = require("../utils/factory");
const metadata_1 = require("../constants/metadata");
class Mutation {
    devour({ commands, options, missing, majors, }) {
        const command = this.findCommand(commands);
        const optionDatas = filter.optionsToDatas(options);
        const globalOptions = filter.optionsToKeyValue(optionDatas);
        const commandOptions = this.findSubOptions(command);
        const nextOptions = command ? commandOptions : globalOptions;
        this.args = arg_1.default(nextOptions, { permissive: true });
        let currnetTriggerOptionKey = '';
        const nativeOption = Object.keys(nextOptions)
            .reduce((pre, key) => {
            if (!key.startsWith('--'))
                return pre;
            const nativeKey = filter.removeHyphen(key);
            const nativeVal = this.args[key] || undefined;
            if (nativeVal) {
                currnetTriggerOptionKey = nativeKey;
            }
            return Object.assign({}, pre, { [nativeKey]: nativeVal });
        }, {});
        const factory = new factory_1.Factory({
            nativeOption,
            args: this.args,
            commands,
            options,
        });
        if (command) {
            const params = factory.getServiceParams(command);
            return new command(...params);
        }
        const currentTriggerOptionFn = options.find(fn => {
            const data = Reflect.getMetadata(metadata_1.metadata.OPTION_IDENTIFIER, fn);
            return data && data.name === currnetTriggerOptionKey;
        });
        if (currentTriggerOptionFn) {
            const value = nativeOption[currnetTriggerOptionKey];
            const params = factory.getServiceParams(currentTriggerOptionFn, value);
            return new currentTriggerOptionFn(...params);
        }
        if (this.args._.length > 0) {
            return missing.forEach(missCommand => {
                const params = factory.getServiceParams(missCommand);
                new missCommand(...params);
            });
        }
        majors.forEach(majorCommand => {
            const params = factory.getServiceParams(majorCommand);
            new majorCommand(...params);
        });
    }
    findSubOptions(command) {
        if (!command)
            return {};
        const subs = Reflect.getMetadata(metadata_1.metadata.SUB_OPTION_IDENTIFIER, command);
        return filter.optionsToKeyValue(subs);
    }
    findCommand(commands) {
        const inputs = process.argv.slice(2) || [];
        const first = inputs[0];
        if (!first || first.startsWith('-'))
            return undefined;
        return commands.find(item => {
            const data = Reflect.getMetadata(metadata_1.metadata.COMMAND_IDENTIFIER, item);
            if (!data)
                return false;
            return data.name === first || data.alias === first;
        });
    }
}
exports.Mutation = Mutation;
